#ifndef _LT_CAMERA_H_
#define _LT_CAMERA_H_

#include "LT_math.h"
#include "LT_vector.h"

class cCamera
{
	public:
		inline void Setup(float xx, float yy, float zz, float d, int h, int e)
		{
			mDistance = d;
			mHeading = h;
			mElevation = e;
			
			mTarget = cVector3D(xx, yy, zz);
			
			if(mElevation > 89)
			{
				mElevation = 89;
			}
			else if(mElevation < -89)
			{
				mElevation = -89;	
			}
			
			float cosE = Math::abs(Math::cos(mElevation));			
			
			mPosition.x = Math::sin(mHeading)*cosE;
			mPosition.z = Math::cos(mHeading)*cosE;
			mPosition.y = Math::sin(mElevation);
			
			mPosition = mTarget + mPosition*mDistance;
			
		};
			
		inline void SetTarget(cVector3D t){ mTarget = t; };
		inline void SetDistance(float d){ mDistance = d; };
		
		inline void Turn(int h, int e)
		{
			mHeading -= h;
			mElevation += e;
						
			if(mElevation > 89)
			{
				mElevation = 89;
			}
			else if(mElevation < -89)
			{
				mElevation = -89;	
			}
		};
		
		inline void Update(void)
		{
			float cosE = Math::abs(Math::cos(mElevation));
			
			mPosition.x = Math::sin(mHeading)*cosE;
			mPosition.z = Math::cos(mHeading)*cosE;
			mPosition.y = Math::sin(mElevation);
			
			mPosition = mTarget + mPosition*mDistance;
		};
		
	public:	
		int mHeading;
		int mElevation;
		float mDistance;
		cVector3D mPosition;
		cVector3D mTarget;	
};

#endif //_LT_CAMERA_H_
