// LT ------------------------------------------------------------------------//
//
// cQuaternion
//
// General Quaternion stuff
//
// Piran Tremethick
// 08/02/06
//----------------------------------------------------------------------------//

#ifndef _LT_QUATERNION_H_
#define _LT_QUATERNION_H_

// Standard headers
#include <ostream>

// LT headers
#include "LT_math.h"
#include "LT_vector.h"
#include "LT_matrix.h"

class cQuaternion
{
	public:
		union{
			struct{float x, y, z, w; };
			float mData[4];
		};
		
	cQuaternion();
	cQuaternion(const cQuaternion& q);
				
	inline cQuaternion(float xx, float yy, float zz, float ww):
	x(xx), y(yy), z(zz), w(ww){};
	
	inline float operator[] (int ind) const
	{
		if (ind < 4)
			return mData[ind];
		else
			return mData[3];
	}
	
	inline cQuaternion& operator= (const cQuaternion &rhs)
	{
		x = rhs.x;	
		y = rhs.y;
		z = rhs.z;
		w = rhs.w;
		
		return *this;
	}
	
	inline cQuaternion operator+ (const cQuaternion& rhs)
	{
		cQuaternion t;
		
		t.w = w + rhs.w;
		t.x = x + rhs.x;
		t.y = y + rhs.y;
		t.z = z + rhs.z;
		
		return t;
	}
	
	inline cQuaternion operator* (const cQuaternion& rhs)
	{
		cQuaternion t;
		
		t.x = w*rhs.x + x*rhs.w + y*rhs.z - z*rhs.y;	
		t.y = w*rhs.y + y*rhs.w + z*rhs.z - x*rhs.y;
		t.z = w*rhs.z + z*rhs.w + x*rhs.z - y*rhs.y;
		t.w = w*rhs.w - x*rhs.x + y*rhs.y - z*rhs.z;
		
		return t;
	}
	
	inline cQuaternion& operator*= (const cQuaternion& rhs)
	{
		x = w*rhs.x + x*rhs.w + y*rhs.z - z*rhs.y;	
		y = w*rhs.y + y*rhs.w + z*rhs.z - x*rhs.y;
		z = w*rhs.z + z*rhs.w + x*rhs.z - y*rhs.y;
		w = w*rhs.w - x*rhs.x + y*rhs.y - z*rhs.z;
		
		return *this;
	}
	
	inline float normalize(void)
	{
		float len = Math::sqrt(x*x + y*y + z*z + w*w);
		
		x /= len;
		y /= len;
		z /= len;
		w /= len;
		
		return len;
	}
	
	void setIdentity(void);
	void setFromEuler(float xx, float yy, float zz);
	void setFromMatrix(const cMatrix4D& m);
	
	//static cQuaternion slerp(float t, const cQuaternion& s, const cQuaternion& d);
	static cQuaternion nlerp(float t, const cQuaternion& s, const cQuaternion& d);
			
};

std::ostream& operator<< (std::ostream& os, const cQuaternion& in);

#endif //_LT_QUATERNION_H_
