// LT ------------------------------------------------------------------------//
//
// cVector
//
// General Vector stuff
//
// Piran Tremethick
// 08/02/06
//----------------------------------------------------------------------------//

#ifndef _LT_VECTOR_H_
#define _LT_VECTOR_H_

// Standard Headers
#include <ostream>

//LT headers
#include "LT_math.h"

class cVector3D
{
public:
	union{
		struct{ float x, y, z; };
		float mData[3];
	};

	cVector3D();
	inline cVector3D(const cVector3D& in):x(in.x), y(in.y), z(in.z){}
	inline cVector3D(float xx, float yy, float zz):x(xx), y(yy), z(zz){}
	
	inline float operator[] (int ind) const
	{
		if (ind < 3)
			return mData[ind];
		else
			return mData[2];
	}
		
	inline cVector3D& operator= (const cVector3D &rhs)
	{
		x = rhs.x;	
		y = rhs.y;
		z = rhs.z;
		
		return *this;
	}
	
	inline cVector3D operator+ (const cVector3D &rhs)
	{
		cVector3D tmp;
		
		tmp.x = x + rhs.x;	
		tmp.y = y + rhs.y;
		tmp.z = z + rhs.z;
		
		return tmp;
	}
	
	inline cVector3D operator- (const cVector3D &rhs)
	{
		cVector3D tmp;
		
		tmp.x = x - rhs.x;	
		tmp.y = y - rhs.y;
		tmp.z = z - rhs.z;
		
		return tmp;
	}
	
	inline cVector3D operator* (float rhs)
	{
		cVector3D tmp;
		
		tmp.x = x * rhs;	
		tmp.y = y * rhs;
		tmp.z = z * rhs;
		
		return tmp;
	}

	inline cVector3D operator/ (float rhs)
	{
		cVector3D tmp;
		
		tmp.x = x / rhs;	
		tmp.y = y / rhs;
		tmp.z = z / rhs;
		
		return tmp;
	}
	
	inline cVector3D& operator+= (const cVector3D &rhs)
	{
		x += rhs.x;	
		y += rhs.y;
		z += rhs.z;
		
		return *this;
	}
	
	inline cVector3D& operator-= (const cVector3D &rhs)
	{
		x -= rhs.x;	
		y -= rhs.y;
		z -= rhs.z;
		
		return *this;
	}
	
	inline cVector3D& operator*= (float rhs)
	{
		x *= rhs;	
		y *= rhs;
		z *= rhs;
		
		return *this;
	}
	
	inline cVector3D& operator/= (float rhs)
	{
		x /= rhs;	
		y /= rhs;
		z /= rhs;
		
		return *this;
	}
	
	inline bool operator< (const cVector3D &rhs) const
	{
		if(x<rhs.x && y<rhs.y && z<rhs.z)
			return true;
		else
			return false;	
	}
	
	inline bool operator> (const cVector3D &rhs) const
	{
		if(x>rhs.x && y>rhs.y && z>rhs.z)
			return true;
		else
			return false;	
	}
	
	inline float length(void) const
	{
		return Math::sqrt( (x*x) + (y*y) + (z*z) );	
	}
	
	inline float lengthManHat(void) const
	{
		return (x*x) + (y*y) + (z*z);	
	}
	
	inline float dot(const cVector3D &rhs) const
	{
		return (x*rhs.x) + (y*rhs.y) + (z*rhs.z);	
	}
	
	inline cVector3D cross(const cVector3D &rhs) const
	{
		cVector3D tmp;
		
		tmp.x = y*rhs.z - z*rhs.y;
		tmp.y = z*rhs.x - x*rhs.z;
		tmp.z = x*rhs.y - y*rhs.x;
		
		return tmp;
	}
	
	inline float normalize(void)
	{
		float len = Math::sqrt(x*x + y*y + z*z);
		
		x /= len;
		y /= len;	
		z /= len;
		
		return len;
	}

};

std::ostream& operator<< (std::ostream &os, const cVector3D& in);
	

#endif //_LT_VECTOR_H_
