#include <vector>
#include <algorithm>
#include <iostream>

#include "LT_sdl.h"
#include "LT_actor.h"
#include "LT_quaternion.h"
#include "LT_bone.h"

cActor::cActor()
{
	mTime = 0;
	mHeading = 0;
	mBlend = 250;
	mCurrentAction = 0;
	mWalkCycle = 0;
	mNextAction = 1;
}

void cActor::SetConstraints(void)
{
	std::vector<cBone>::iterator biter;

	biter = find(mArmature.mBones.begin(), mArmature.mBones.end(), "Bone.008");
	mLeftFoot.Setup(&(*biter), 0.0, 1.0, 0.0);
	biter = find(mArmature.mBones.begin(), mArmature.mBones.end(), "Bone.007");
	mRightFoot.Setup(&(*biter), 0.0, 1.0, 0.0);
}

void cActor::UpdateArmature(void)
{
	if(mArmature.mRootBone != NULL)
		mArmature.Update();
}

void cActor::DrawArmature(unsigned int mode)
{
	if(mArmature.mRootBone != NULL)
		mArmature.Draw(mode);
}

void cActor::Animate(unsigned int t)
{
	mTime += t;
	cMatrix4D m;
	cQuaternion q;

	if(mTime > mAction[mCurrentAction].mLength)
		mTime = 34; //<- there is no key at time 0, not a nice hack, should be fixed in exporter

	mPose.Set(mArmature);
	mPose2.Set(mArmature);

	//Get the pose this frame from the animation
	mAction[mCurrentAction].EvaluateAt(mTime, mPose);
	
	if(mNextAction != mCurrentAction)
	{
		/*mBlend += t;
		if(mBlend > 250)
		{
			mCurrentAction = mNextAction;
			mBlend = 34;
		}
		else
		{
			mAction[mNextAction].EvaluateAt(0, mPose2);
			mPose.Blend(static_cast<float>(mBlend)/250.0, mPose2);
		}*/
		
		//Blend not quite working just yet unfortunately
		mCurrentAction = mNextAction;
	}

	//The setFromEuler method for quaternions is incorrect, for now need to make a matrix
	//and then set the quaternion from that, not nice but easy enough to fix when I have time.
	m.setFromEuler(0.0, 0.0, mHeading);
	q.setFromMatrix(m);
	mPose.mPoseRoot->mOrientation = q * mPose.mPoseRoot->mOrientation;
	
	ApplyPose(mPose);

	//Evaluate constraints
	mRightFoot.Evaluate(mArmature.mRootBone, mPose);
	mLeftFoot.Evaluate(mArmature.mRootBone, mPose);

	ApplyPose(mPose);
}

void cActor::ApplyPose(cPose &pose)
{
	std::vector<cBone>::iterator itor = pose.mPoseBones.begin();
	std::vector<cBone>::iterator biter = mArmature.mBones.begin();
	std::vector<cBone>::iterator bitend = mArmature.mBones.end();
	
	while(biter != bitend)
	{
		(*biter).mTransform = (*itor).mTransform;
		(*biter).mOrientation = (*itor).mOrientation;
		++itor;
		++biter;	
	}
	
	mArmature.Update();	
}
