#include <vector>
#include <algorithm>

#include "LT_iksolver.h"
#include "LT_armature.h"
#include "LT_math.h"
#include "LT_matrix.h"
#include <iostream>
#include "LT_sdl.h"

void cIK::Setup(cArmature& arm)
{
	//Link to first bone in the IK chain
	std::vector<cBone>::iterator itor = find(arm.mBones.begin(), arm.mBones.end(), mIKBones[0]->mName);
	mIndices[0] = distance(arm.mBones.begin(), itor);

	//Link to root bone in the IK chain
	itor = find(arm.mBones.begin(), arm.mBones.end(), mIKBones[0]->mParent->mName);
	mIKBones[1] = &(*itor);
	mIndices[1] = distance(arm.mBones.begin(), itor);
}

void cIK::EvaluateAt(unsigned int t, cAction &action)
{
	/*//Update blend factor between this pose and IK pose
	//	if blend factor > 1: reset and get pose at next keyframe
	//if foot position in IK pose is invalid: move the IK target
	//blend between this pose and IK pose by blend factor

	//Get the two keyframes bounding the current time
	action.GetKeyFrames();
	//if ik target pose at second keyframe is not valid
	action.EvaluateAt(second_key_frame_time, IKpose);
	IKpose.ApplyPose();
	get target in world space
		if target intersects any bounding boxes
			move target up in Y by bounding box height-target.y;
			solve IK chain
	//		move ik target and solve
	//calculate blend factor between these keyframes
	blend factor = (t - key1.time) / (key2.time - key1.time); 
	//blend between current animated pose and IK pose by blend factor
	pose.blend(IKpose);*/
	
	//Get the next key pose
	//action.mChannels[mIndices[0]].EvaluateAt(950, pose.mPoseBones[mIndices[0]].mOrientation);
	//action.mChannels[mIndices[1]].EvaluateAt(950, pose.mPoseBones[mIndices[1]].mOrientation);

	//IK solver
	unsigned int tries = 0;
	unsigned int i = 0;
		
	//CCD as described by Jeff Lander
	cVector3D root, effector, totarget, toeffector, cross, diff;
	cQuaternion orient;
	float cosAngle, turnAngle;
	unsigned int link;

	link = 0;
	tries = 0;
	do
	{
		//Get the root position of the current bone
		root = mIKBones[link]->mWorldTransform.getTransform();

		//Get the position of the effector
		effector.x = effector.z = 0.0;
		effector.y = mIKBones[0]->mLength;
		effector = mIKBones[0]->mWorldTransform*effector;

		//Check if close enough already
		diff = effector - mTarget;
		if (diff.length() > IK_THRESHOLD)
		{
			//Vector from root to effector
			toeffector = effector-root;
			toeffector.z = 0.0;
			//Vector from the root to the target
			totarget = mTarget-root;
			totarget.z = 0.0;

			//Normalize both
			toeffector.normalize();
			totarget.normalize();

			//Dot product to find the cosine of the angle between
			cosAngle = totarget.dot(toeffector);

			//Don't need to do anything if already pointing at target
			if (cosAngle < 0.999)
			{
				//Do a cross product to check which way to rotate
				cross = toeffector.cross(totarget);
				if (cross.z >= 0.0f)	//rotate clockwise
				{
					turnAngle = Math::invcos(cosAngle);
					/*if(turnAngle < IK_MIN_ANGLE)
						turnAngle = IK_MIN_ANGLE;*/
				
					orient.setFromEuler(0.0, 0.0, -turnAngle);	//Create the offset to the orientation
				}
				else if (cross.z < 0.0f)	//anti clockwise
				{
					turnAngle = Math::invcos(cosAngle);
					/*if(turnAngle > IK_MAX_ANGLE)
						turnAngle = IK_MAX_ANGLE;*/
                    
					orient.setFromEuler(0.0, 0.0, turnAngle);
				}
				//update matrices
				mIKBones[link]->mOrientation = mIKBones[link]->mOrientation+orient;
				mIKBones[link]->mOrientation.normalize();
				mIKBones[link]->Update();
			}
			if (++link > 1) link = 0;	//Reached the root, start over at the end
		}
	//Quit when close enough, or looped a lot (prevent infinite looping for unreachable targets)
	effector.x = effector.z = 0.0;
	effector.y = mIKBones[0]->mLength;
	effector = mIKBones[0]->mWorldTransform*effector;

	mEffector = effector;

	diff = mTarget-effector;
	} while (tries++ < IK_MAX_TRIES && diff.length() > IK_THRESHOLD);
}
