#include <iostream>
#include <sstream>
#include <string>
#include <vector>
#include <algorithm>

#include "tinyxml.h"
#include "LT_app.h"
#include "LT_actor.h"
#include "LT_quaternion.h"

void LoadMatrix(TiXmlElement* xml, cMatrix4D &m)
{
	TiXmlText* s;
	std::string str;
	std::string err("XML ERROR: ");
	std::stringstream strm;
	int num, ind;
	
	if(!(s = xml->FirstChild()->ToText()))
		throw err + "Expecting 'matrix' to have data.";
	
	if(s->Type() != TiXmlNode::TEXT)
		throw err + "Expecting 'matrix' data to be a 'text' node.";
	
	str = s->Value();
	ind = num = 0;

	strm.clear();
	num = str.find(",", ind);
	strm << str.substr(ind, num);
	strm >> m.m[0][0];

	strm.clear();
	ind = num+1;
	num = str.find(",", ind);
	num -= ind;				
	strm << str.substr(ind, num);
	strm >> m.m[1][0];
	
	strm.clear();
	ind = ind+num+1;
	num = str.find(",", ind);
	num -= ind;				
	strm << str.substr(ind, num);
	strm >> m.m[2][0];
	
	strm.clear();
	ind = ind+num+1;
	num = str.find(",", ind);
	num -= ind;				
	strm << str.substr(ind, num);
	strm >> m.m[3][0];
	
	
	strm.clear();
	ind = ind+num+1;
	num = str.find(",", ind);
	num -= ind;				
	strm << str.substr(ind, num);
	strm >> m.m[0][1];
	
	strm.clear();
	ind = ind+num+1;
	num = str.find(",", ind);
	num -= ind;				
	strm << str.substr(ind, num);
	strm >> m.m[1][1];
	
	strm.clear();
	ind = ind+num+1;
	num = str.find(",", ind);
	num -= ind;				
	strm << str.substr(ind, num);
	strm >> m.m[2][1];
	
	strm.clear();
	ind = ind+num+1;
	num = str.find(",", ind);
	num -= ind;				
	strm << str.substr(ind, num);
	strm >> m.m[3][1];
	
	
	strm.clear();
	ind = ind+num+1;
	num = str.find(",", ind);
	num -= ind;				
	strm << str.substr(ind, num);
	strm >> m.m[0][2];
	
	strm.clear();
	ind = ind+num+1;
	num = str.find(",", ind);
	num -= ind;				
	strm << str.substr(ind, num);
	strm >> m.m[1][2];
	
	strm.clear();
	ind = ind+num+1;
	num = str.find(",", ind);
	num -= ind;				
	strm << str.substr(ind, num);
	strm >> m.m[2][2];
	
	strm.clear();
	ind = ind+num+1;
	strm << str.substr(ind);
	strm >> m.m[3][2];
}

void LoadActorFromXML(const char* fname, cActor &actor)
{
	std::string error("XML ERROR: ");
	TiXmlDocument file(fname);
	try
	{
		if(!file.LoadFile())
			throw error + file.ErrorDesc();
	
		TiXmlHandle handle(&file);
		TiXmlElement* element = NULL;
		double bones;
	
		if(!(element = handle.FirstChild("LTArmature").FirstChild("armature").Element()))
			throw error + "Expecting 'armature' tag.";
		
		if(!element->Attribute("name"))
			throw error + "Expecting 'name' attribute for 'armature'.";
			
		actor.mName = element->Attribute("name");
		
		if(!(element = element->FirstChildElement("skeleton")))
			throw error + "Expecting 'skeleton' tag.";
		
		if(!element->Attribute("bones"))
			throw error + "Expecting 'bones' attribute for 'skeleton'.";
			
		element->Attribute("bones", &bones);
		actor.mArmature.mBones.resize(static_cast<int>(bones));
		actor.mPose.mPoseBones.resize(static_cast<int>(bones));
		actor.mPose2.mPoseBones.resize(static_cast<int>(bones));
			
		if(!(element = element->FirstChildElement("bone")))
			throw error + "Expecting (at least one) 'bone' tag.";
		
		std::vector<cBone>::iterator itor = actor.mArmature.mBones.begin();	
		cBone tbone;
		std::string p;
		double l;
				
		for( ; element; element = element->NextSiblingElement(), ++itor)
		{
			if(!element->Attribute("name"))
				throw error + "Expecting 'name' attribute for 'bone'.";
			(*itor).mName = element->Attribute("name");
			
			if(!element->Attribute("len"))
				throw error + "Expecting 'len' attribute for 'bone'.";
			element->Attribute("len", &l);
			(*itor).mLength = l;
			
			LoadMatrix(element->FirstChildElement("matrix"), (*itor).mRestTransform);
			
			(*itor).mTransform = (*itor).mRestTransform;
		}
		
		//Heirachy
		
		if(!(element = handle.FirstChild("LTArmature").FirstChild("armature").FirstChild("heirachy").Element()))
			throw error + "Expecting 'heirachy' tag.";
			
		if(!(element = element->FirstChildElement("node")))
			throw error + "Expecting (at least one) 'node' tag.";
			
		std::string bname;
		cMatrix4D r;
		
		for( ; element; element = element->NextSiblingElement())
		{
			if(!element->Attribute("name"))
				throw error + "Expecting 'name' attribute for 'node'.";
			
			itor = find(actor.mArmature.mBones.begin(), actor.mArmature.mBones.end(), element->Attribute("name"));
					
			if(!element->FirstChild()->ToText())
				throw error + "Expecting a 'text' parent name for 'node'.";
			
			p = element->FirstChild()->ToText()->Value();
			if(p == "NULL")
			{
				(*itor).mParent = NULL;
				actor.mArmature.mRootBone = &(*itor);
				r.setFromEuler(-90, 0, 0);
				(*itor).mRestTransform = r*(*itor).mRestTransform;
				(*itor).mTransform = (*itor).mRestTransform;
			}
			else
			{
				(*itor).mParent = &(*(find(actor.mArmature.mBones.begin(), actor.mArmature.mBones.end(), p)));
				(*itor).mParent->mChildren.push_back(&(*itor));	
			}
		}
				
		//Pose might not be needed...
		actor.mPose.Setup(actor.mArmature);
		actor.mPose2.Setup(actor.mArmature);

		if(!(element = handle.FirstChild("LTArmature").FirstChild("actions").FirstChild("action").Element()))
			throw error + "Expecting 'action' tag.";
		
		ChannelIterator citor = actor.mAction[0].mChannels.begin();
		KeyFrameIterator kitor;
		TiXmlElement* channel = NULL;
		TiXmlElement* keys = NULL;
					
		unsigned int i = 0;
		for( ; element; element = element->NextSiblingElement(), ++i)
		{
			if(!element->Attribute("name"))
				throw error + "Expecting 'name' attribute for 'action'.";
			
			actor.mAction[i].mName = element->Attribute("name");
			
			if(!element->Attribute("len"))
				throw error + "Expecting 'length' attribute for 'action'.";
				
			element->Attribute("len", &l);
			actor.mAction[i].mLength = static_cast<unsigned int>(l);
			
			actor.mAction[i].mChannels.resize(actor.mArmature.mBones.size());
			
			if(!(channel = element->FirstChildElement("channel")))
				throw error + "Expecting 'channel' tag.";
			
			citor = actor.mAction[i].mChannels.begin();
			
			for( ; channel; channel = channel->NextSiblingElement(), ++citor)
			{
				//if(!channel->Attribute("name"))
				//	throw error + "Expecting 'name' attribute for 'channel'.";
			
				//(*citor).mName = channel->Attribute("name");
				
				if(!(keys = channel->FirstChildElement("keyframes")))
					throw error + "Expecting 'keyframes' tag.";
				
				if(!keys->Attribute("keys"))
					throw error + "Expecting 'keys' attribute for 'keyframes'.";
					
				keys->Attribute("keys", &l);
				(*citor).mKeyFrames.resize(static_cast<unsigned int>(l));
				
				kitor = (*citor).mKeyFrames.begin();
				
				if(!(keys = keys->FirstChildElement("key")))
					throw error + "Expecting (at least one) 'key' tag.";
				
				kitor = (*citor).mKeyFrames.begin();
				
				for( ; keys; keys = keys->NextSiblingElement("key"), ++kitor)
				{
					if(!keys->Attribute("t"))
						throw error + "Expecting 't' attribute for 'key'.";
						
					keys->Attribute("t", &l);
					(*kitor).mTime = static_cast<unsigned int>(l);
					
					if(!keys->FirstChildElement("quat"))
						throw error + "Expecting 'quat' tag for 'key'.";
						
					if(!keys->FirstChildElement("quat")->Attribute("w", &l))
						throw error + "Expecting 'w' attribute for 'quat'.";
						
					(*kitor).mValue.w = l;
					
					if(!keys->FirstChildElement("quat")->Attribute("x", &l))
						throw error + "Expecting 'x' attribute for 'quat'.";
						
					(*kitor).mValue.x = l;
					
					if(!keys->FirstChildElement("quat")->Attribute("y", &l))
						throw error + "Expecting 'y' attribute for 'quat'.";
						
					(*kitor).mValue.y = l;
					
					if(!keys->FirstChildElement("quat")->Attribute("z", &l))
						throw error + "Expecting 'z' attribute for 'quat'.";
						
					(*kitor).mValue.z = l;
				}
			}
		}
	}
	catch(std::string &err)
	{
		cLTApp::ExitError(err.c_str());
	}
}
